require "rails_connector/core_extensions.rb"
require "rails_connector/date_attribute.rb"
require "active_support/core_ext/numeric/time"

# require "infopark_component_cache/guards/always_consistent"
# require "infopark_component_cache/guards/never_consistent"
require "infopark_component_cache/guards/value_present"
require "infopark_component_cache/guards/cms_state_guard"

require "infopark_component_cache/guards/last_changed"
require "infopark_component_cache/guards/obj_count"
require "infopark_component_cache/guards/valid_from"
require "infopark_component_cache/guards/valid_until"

require "infopark_component_cache/guards/delayed_last_changed"
require "infopark_component_cache/guards/delayed_obj_count"
require "infopark_component_cache/guards/delayed_valid_from"
require "infopark_component_cache/guards/delayed_valid_until"

require "infopark_component_cache/key_generator"
require "infopark_component_cache/consistency_guard"
require "infopark_component_cache/cache_storage"
require "infopark_component_cache/component"
require "infopark_component_cache/component_cache"

module InfoparkComponentCache
  # @!scope class
  # This parameter should be initialized to the root object
  # class of the project. It defaults to "::Obj".
  mattr_accessor :obj_root_class

  # @author Tomasz Przedmojski <tomasz.przedmojski@infopark.de>
  class Engine < Rails::Engine
    initializer "component_cache.helpers" do
      [
        InfoparkComponentCacheHelper
      ].each do |helper|
        ActionView::Base.__send__(:include, helper)
      end
    end

    initializer "set default obj_root_class" do
      InfoparkComponentCache.obj_root_class ||= ::Obj
      CmsStateGuard.obj_root_class = InfoparkComponentCache.obj_root_class
    end
  end
end
