require "infopark_component_cache/guards/cms_state_guard"

module InfoparkComponentCache
  module Guards
    # @author Tomasz Przedmojski <tomasz.przedmojski@infopark.de>
    #
    # This Guard class ensures that the objects in database do not change inbetween.
    # (It caches the newest timestamp and compares it to the current value)
    class LastChanged < CmsStateGuard
      def consistent?
        last_changed_known? && no_changes_since?
      end

      def guard!
        cache.write(cache_key, current_last_changed)
      end

      # @return true if a timestamp can be read from cache with {#cache_key}
      def last_changed_known?
        cache.exist?(cache_key) && cache.read(cache_key).kind_of?(Time)
      end

      # @return true if no obj has been changed since last {#guard!}
      def no_changes_since?
        current_last_changed <= cache.read(cache_key)
      end

      # @return [String] the cache key for storing {#current_last_changed}
      def cache_key
        component.cache_key("last_changed")
      end

      # @return [Time] the timestamp of the most recent change to any Obj
      def current_last_changed
        str_value = scoped_relation.maximum(:last_changed)
        return str_value if str_value.kind_of? Time

        RailsConnector::DateAttribute.parse(str_value)
      end
    end
  end
end
